/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package StoneBreak;

import StoneBreak.StoneBreakGame.Blocks.BlockParticle;
import StoneBreak.StoneBreakGame.Globals;
import StoneBreak.StoneBreakGame.GameEngine;
import StoneBreak.StoneBreakGame.Paddle.MOVE_DIRECTION;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Panel;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.util.Timer;
import java.util.TimerTask;
import javax.swing.JOptionPane;

/**
 * GamePanel is the main container of the game. Everything is painted on this panel.
 * GamePanel calls periodically the main game loop.
 * @author Sunshine2k
 */
public class GamePanel extends Panel implements Runnable, KeyListener, MouseMotionListener {

    private GameEngine  gameEngine;
    private Thread      mainThread;

    // doublebuffer to avoid flickering
    private Image       doubleBufImage = null;
    private Graphics2D  gDoubleBuf;

    Timer focusTimer;

    public GamePanel()
    {
        this.setBounds(0, 0, Globals.GAMEPANEL_WIDTH, Globals.GAMEPANEL_HEIGHT);

        gameEngine = new GameEngine();
        gameEngine.Init();
        
        this.addKeyListener(this);
        this.addMouseMotionListener(this);

        this.addMouseListener(new MouseAdapter() {

            @Override
            public void mouseClicked(MouseEvent me)
            {
                gameEngine.HandleMouseClick(me);
            }
        });

        // hide cursor by replacing the cursor with a transparant image
        BufferedImage cursorImg = new BufferedImage(16, 16, BufferedImage.TYPE_INT_ARGB);
        Cursor blankCursor = Toolkit.getDefaultToolkit().createCustomCursor(
            cursorImg, new Point(0, 0), "blank cursor");
        this.setCursor(blankCursor);

        // start focus timer to request focus after applet is loaded
        TimerTask task = new TimerTask()
        {
            public void run()
            {
                requestFocusInWindow();
            }
        };
        focusTimer = new Timer();
        focusTimer.schedule(task, 250);

        // start main timer
        mainThread = new Thread(this);
        mainThread.start();


    }

    @Override
    public void update(Graphics g)
    {
        
        paint(g);
    }
    
    @Override
    public void paint(Graphics g)
    {
        if (doubleBufImage == null)
        {
            // the doublebuffer image cannot be created in the constructor because
            // the panel is not visible in this moment and the call fails.
            doubleBufImage = createImage(Globals.GAMEPANEL_WIDTH, Globals.GAMEPANEL_HEIGHT);
            gDoubleBuf = (Graphics2D)doubleBufImage.getGraphics();
        }
        // clear background
        gDoubleBuf.setColor(Color.darkGray.brighter());
        gDoubleBuf.fillRect(0, 0, Globals.GAMEPANEL_WIDTH, Globals.GAMEPANEL_HEIGHT);

        gameEngine.MainLoop(gDoubleBuf);

        // blit memory image to screen
        g.drawImage(doubleBufImage, 0, 0, null);
    }

    /**
     * Thread that handles the main game loop.
     */
    public void run()
    {
        long startTime;
        
        while (true)
        {
            startTime = System.currentTimeMillis();
            repaint();
            try
            {
                Thread.sleep(
                        (long)Math.max(
                        (long)(1000.0 / Globals.FPS_TARGET) - (long)(System.currentTimeMillis() - startTime), 0));
            }
            catch (Exception ex)
            {
                /* should not happen - what to do here */
            }
        }


    }

    /**
     * Does nothing - only to implement the interface.
     * @param e
     */
    public void keyTyped(KeyEvent e)
    {
        /* nothing to do */
    }

    /**
     * Handles all key presses.
     * @param e KeyEvent
     */
    public void keyPressed(KeyEvent e)
    {
        if (e.getKeyCode() == KeyEvent.VK_RIGHT)
        {
            gameEngine.PlayerPaddleMoveOneStep(MOVE_DIRECTION.RIGHT);
        }
        else if (e.getKeyCode() == KeyEvent.VK_LEFT)
        {
            gameEngine.PlayerPaddleMoveOneStep(MOVE_DIRECTION.LEFT);
        }

        /* hidden keys */
        else if (e.getKeyCode() == KeyEvent.VK_ADD)
        {
            gameEngine.incBallSpeed();
        }
        else if (e.getKeyCode() == KeyEvent.VK_SUBTRACT)
        {
            gameEngine.decBallSpeed();
        }
        else if (e.getKeyCode() == KeyEvent.VK_NUMPAD9)
        {
            gameEngine.beamToNextLevel();
        }

        else if (e.getKeyCode() == KeyEvent.VK_NUMPAD2)
        {
            BlockParticle.IncParticleSize();
        }
        else if (e.getKeyCode() == KeyEvent.VK_NUMPAD1)
        {
            BlockParticle.DecParticleSize();
        }

        else if (e.getKeyCode() == KeyEvent.VK_I)
        {
            JOptionPane.showMessageDialog(this,
                    "StoneBreak 1.0 - www.sunshine2k.de", "About...", JOptionPane.INFORMATION_MESSAGE);
        }

       
    }

    public void keyReleased(KeyEvent e)
    {
      /* nothing to do */
    }

    public void mouseDragged(MouseEvent e)
    {
        /* nothing to do - function required due to MouseMotionListener interface implementation */
    }

    public void mouseMoved(MouseEvent e)
    {
        gameEngine.PlayerPaddleNewPosition(e.getX());
    }



}
