/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package StoneBreak.StoneBreakGame;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RadialGradientPaint;
import java.awt.geom.Point2D;

/**
 *
 * @author Sunshine2k
 */
 class Ball {

    /* current position of ball - middle point of circle, not topleft! */
    private double      x, y;

    /* current speed of ball */
    private double      speedX, speedY;

    /* actual color of the ball */
    private Color   ballColor;

    /**
     * Default constructor.
     */
    public Ball()
    {
        this(0, 0);
    }

    /**
     * Constructor with setting intial position of the ball.
     * @param posX x-coordinate in pixels of start position of the ball
     * @param posY y-coordinate in pixels of start position of the ball
     */
    public Ball(int posX, int posY)
    {
        x = posX;
        y = posY;
        speedX = speedY = 0;
        ballColor = Globals.BALL_COLOR;
    }

    /**
     * Draw actual ball.
     * @param g Graphics object where to draw the ball on.
     */
    public void paint(Graphics2D g)
    {
        Color oldColor = g.getColor();
        g.setColor(ballColor);

        // draw ball with light spot coming from topleft
        Point2D center = new Point2D.Float((float)(x-(Globals.BALL_RADIUS_STD>>1)), (float)(y-(Globals.BALL_RADIUS_STD>>1)));
        float radius = Globals.BALL_RADIUS_STD*1.5f;
        float[] dist = {0.0f, 1.0f};
        Color[] colors = {new Color(220, 220, 220),  ballColor};
        RadialGradientPaint rp = new RadialGradientPaint(center, radius, dist, colors);
        g.setPaint(rp);

        g.fillOval((int)x - Globals.BALL_RADIUS_STD, (int)y - Globals.BALL_RADIUS_STD,
                Globals.BALL_RADIUS_STD << 1, Globals.BALL_RADIUS_STD << 1);

        g.setColor(oldColor);
    }

    /**
     * Check if ball hits the paddle.
     * @param paddle the playerpaddle to check against.
     * @return true if ball hits the paddle, otherwise false.
     */
    public boolean hitPaddle(Paddle paddle)
    {
        return ( ((getRightSideX() >= paddle.getLeftSideX()) && (getLeftSideX() <= paddle.getRightSideX())) ||
                 ((getBottomSideY() >= paddle.getTopSideY()) && (getTopSideY() <= paddle.getBottomSideY())) );
    }


    public static void DrawBall(Graphics2D g, int x, int y, int radius)
    {
        Point2D center = new Point2D.Float(x, y);
        float radius2 = Globals.BALL_RADIUS_STD*1.5f;
        float[] dist = {0.0f, 1.0f};
        Color[] colors = {new Color(210, 210, 210), Globals.BALL_COLOR};
        RadialGradientPaint rp = new RadialGradientPaint(center, radius2, dist, colors);
        g.setPaint(rp);

        g.fillOval((int)x - radius, (int)y - radius,
               radius << 1, radius << 1);
    }

    /**
     * Resets the ball color to default.
     */
    public void setDefaultColor()
    {
        ballColor = Globals.BALL_COLOR;
    }

    /**
     * Set ball color.
     * @param c new color
     */
    public void setBallColor(Color c)
    {
        ballColor = c;
    }
    
    /**
     * Set the x-position of the ball
     * @param x the new x-coordinate in pixels
     */
    public void setPosX(double x)
    {
        this.x = x;
    }
    /**
     * Set the y-position of the ball
     * @param y the new y-coordinate in pixels
     */
    public void setPosY(double y) { this.y = y; }
    /**
     * Set new speed in x-direction of the ball
     * @param speedX the new x-speed value
     */
    public void setSpeedX(double speedX) { this.speedX = speedX; }
    /**
     * Set new speed in y-direction of the ball
     * @param speedY the new y-speed value
     */
    public void setSpeedY(double speedY) { this.speedY = speedY; }

    /**
     *
     * @return x-position of the ball
     */
    public double getPosX() { return x; }
    /**
     *
     * @return y-position of the ball
     */
    public double getPosY() { return y; }
    /**
     *
     * @return speed in x-direction of the ball
     */
    public double getSpeedX() { return speedX; }
    /**
     *
     * @return speed in y-direction of the ball
     */
    public double getSpeedY() { return speedY; }

    
    /**
     *
     * @return the x-coordinate of the right side of the ball
     */
    public double getRightSideX()
    {
        return (x + Globals.BALL_RADIUS_STD);
    }
    /**
     *
     * @return the x-coordinate of the left side of the ball
     */
    public double getLeftSideX()
    {
        return (x - Globals.BALL_RADIUS_STD);
    }
    /**
     *
     * @return the y-coordinate of the top side of the ball
     */
    public double getTopSideY()
    {
        return (y - Globals.BALL_RADIUS_STD);
    }
    /**
     *
     * @return the y-coordinate of the bottom side of the ball
     */
    public double getBottomSideY()
    {
        return (y + Globals.BALL_RADIUS_STD);
    }

    /**
     * Change value of x-coordinate of the ball
     * @param v the difference to the old position
     */
    public void incPosX(double v) { x += v; }
    /**
     * Change value of y-coordinate of the ball
     * @param v the difference to the old position
     */
    public void incPosY(double v) { y += v; }

    /**
     * Change speed in x-direction of the ball
     * @param v the factor/multiplier to apply to the old x-speed
     */
    public void incSpeedX(double v) { speedX *= v; }
    /**
     * Change speed in y-direction of the ball
     * @param v the factor/multiplier to apply to the old y-speed
     */
    public void incSpeedY(double v) { speedY *= v; }

}
