/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package StoneBreak.StoneBreakGame.Blocks;

import StoneBreak.StoneBreakGame.Globals;
import java.awt.Color;
import java.awt.Graphics2D;

/**
 * This class represents a normal one-hit block with some 3D bevel-like look.
 * The type of block specifies the color of the block:
 * 1 = blue, 2 = green
 * @author SUN2K
 */
public class BlockTypeStandard extends BlockBaseType {

    /* color used for the normal area of the block */
    private Color color_AreaNormal = new Color(0, 0, 255);
    /* lighter color used for bright left and top border */
    private Color color_LighterTopArea = new Color(90, 90, 255);
    /* darker color used for right and bottom border */
    private Color color_DarkerBottomArea = new Color(0, 0, 210);

    /* size of the broder in pixels - TODO: depending on size! */
    private int bevelSize = 4;

    /* hold two triangle edge points to draw a nice border */
    int[] triangleTopRight_PointsX = new int[3];
    int[] triangleTopRight_PointsY = new int[3];

    int[] triangleBottomLeft_PointsX = new int[3];
    int[] triangleBottomLeft_PointsY = new int[3];


    public enum BLOCKTYPE_STANDARD { BLUE, GREEN };

    /**
     * Constructor for this block, selects a type.
     * @param type 1 = blue, 2 = green
     */
    public BlockTypeStandard(BLOCKTYPE_STANDARD type)
    {
        super();    /* important to call super constructor */

        switch (type)
        {
            case BLUE:     /* BLUE */
            {
                color_AreaNormal = new Color(0, 0, 255);
                color_LighterTopArea = new Color(90, 90, 255);
                color_DarkerBottomArea = new Color(0, 0, 200);
                explosionColor = color_LighterTopArea;
            }
            break;

            case GREEN:
            default:   /* GREEN */
            {
                color_AreaNormal = new Color(0, 160, 0);
                color_LighterTopArea = new Color(90, 220, 90);
                color_DarkerBottomArea = new Color(0, 150, 0);
                explosionColor = color_AreaNormal;
            }
            break;
        }
        //explosionColor = color_AreaNormal;
    }

    /**
     * Override painting method to draw the block at normal state - in other states
     * let superclass do the handling
     * @param g Graphics object used for drawing
     */
    @Override
    public void draw(Graphics2D g)
    {
        if (internalState == BlockState.NORMAL)
        {
            g.setColor(color_AreaNormal);
            g.fillRect(posX, posY, Globals.BLOCKSIZE_X, Globals.BLOCKSIZE_Y);

            g.setColor(color_LighterTopArea);
            g.fillRect(posX, posY, Globals.BLOCKSIZE_X, bevelSize);
            g.fillRect(posX, posY, bevelSize, Globals.BLOCKSIZE_Y);

            g.setColor(color_DarkerBottomArea);
            g.fillRect(posX + Globals.BLOCKSIZE_X - bevelSize, posY, bevelSize, Globals.BLOCKSIZE_Y);
            g.fillRect(posX, posY +  + Globals.BLOCKSIZE_Y - bevelSize, Globals.BLOCKSIZE_X, bevelSize);

            // draw triangles in both edges
            g.setColor(color_LighterTopArea);
            g.fillPolygon(triangleTopRight_PointsX, triangleTopRight_PointsY,  3);
            g.fillPolygon(triangleBottomLeft_PointsX, triangleBottomLeft_PointsY,  3);

            drawBlockBorder(g);
        }
        else
        {
            super.draw(g);
        }
    }

    /**
     * Set x-postion of the block
     * @param newX x-coordinate in pixels
     */
    @Override
    public void setPosX(int newX)
    {
        // call method of upper class to that posX variable gets updated
        super.setPosX(newX);
        // calculate triangle positions with new value
        calcTriangles();
    }

    /**
     * Set y-postion of the block
     * @param newY y-coordinate in pixels
     */
    @Override
    public void setPosY(int newY)
    {
        // call method of upper class to that posX variable gets updated
        super.setPosY(newY);
        // calculate triangle positions with new value
        calcTriangles();
    }

    /**
     * Calculate the coordinates of the small triangles used for drawing a nice
     * border at the topright and lowerleft corner of the block.
     */
    private void calcTriangles()
    {
        // calcluate new points of triangle in topright edge
        triangleTopRight_PointsX[0] = posX + Globals.BLOCKSIZE_X - bevelSize-1;
        triangleTopRight_PointsX[1] = posX + Globals.BLOCKSIZE_X;
        triangleTopRight_PointsX[2] = posX + Globals.BLOCKSIZE_X - bevelSize-1;
        triangleTopRight_PointsY[0] = posY;
        triangleTopRight_PointsY[1] = posY;
        triangleTopRight_PointsY[2] = posY + bevelSize;

        // calculate new points of triangle in bottomleft edge
        triangleBottomLeft_PointsX[0] = posX;
        triangleBottomLeft_PointsX[1] = posX + bevelSize;
        triangleBottomLeft_PointsX[2] = posX;
        triangleBottomLeft_PointsY[0] = posY + Globals.BLOCKSIZE_Y - bevelSize-1;
        triangleBottomLeft_PointsY[1] = posY + Globals.BLOCKSIZE_Y - bevelSize-1;
        triangleBottomLeft_PointsY[2] = posY + Globals.BLOCKSIZE_Y;
    }

    public int getScore()
    {
        return Globals.BLOCKSCORE_TYPE_STANDARD;
    }

}
