/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package StoneBreak.StoneBreakGame;

/**
 *
 * @author SUN2K
 */


/**
 * This class handles the movement and collision of the ball.
 * This is a quick'n dirty hack to make it working; implement later a
 * physically corrct vector-based approach.
 */
class GameBallPhysicsSimple implements GameBallPhysicsInterface {

    private Ball            playerBall;
    private Paddle          playerPaddle;
    private GameLevel       curlevel;

    private double remainingSpeedX, remainingSpeedY;

    /**
     * Main constructor.
     * @param ball
     * @param paddle
     * @param levelManager
     */
    public GameBallPhysicsSimple(Ball ball, Paddle paddle)
    {
        this.playerBall = ball;
        this.playerPaddle = paddle;
    }


    private boolean BallWillHitLeftWall()
    {
        return (playerBall.getLeftSideX() + remainingSpeedX <= 0);
    }

    private boolean BallWillHitRightWall()
    {
        return (playerBall.getRightSideX() + remainingSpeedX >= Globals.GAMEPANEL_WIDTH);
    }

    private boolean BallWillHitTopWall()
    {
        return (playerBall.getTopSideY() + remainingSpeedY <= 0);
    }

    private boolean BallWillHitPaddle()
    {
        return ( (playerBall.getBottomSideY() + remainingSpeedY >= playerPaddle.getPosY()) &&
                 (playerBall.getTopSideY() + remainingSpeedY <= playerPaddle.getPosY() + Globals.PADDLE_HEIGHT) &&
                 (playerBall.getRightSideX() + remainingSpeedX >= playerPaddle.getPosX()) &&
                 (playerBall.getLeftSideX() + remainingSpeedX <= playerPaddle.getPosX() + playerPaddle.getWidth()));
    }

    private boolean BallWillHitLeftSideOfPaddle()
    {
        return ( (playerBall.getRightSideX() <= playerPaddle.getLeftSideX()) &&
                 (playerBall.getRightSideX() + remainingSpeedX >= playerPaddle.getLeftSideX()) );
    }

    private boolean BallWillHitRightSideOfPaddle()
    {
        return ( (playerBall.getLeftSideX() >= playerPaddle.getRightSideX()) &&
                 (playerBall.getLeftSideX() + remainingSpeedX <= playerPaddle.getRightSideX()) );
    }

    /**
     * This function moves the ball and calculates its new position.
     * Handles all collisions with walls, bricks and paddle.
     * TO DO: Improvement by doing physically correct vector-based calculations
     */
    public RESULT_BALLMOVEMENT ProcessBall(GameLevel level)
    {
        RESULT_BALLMOVEMENT result = RESULT_BALLMOVEMENT.NORMAL;

        double doubleTmp, ratio;
        double angle, v;

        curlevel = level;

        remainingSpeedX = playerBall.getSpeedX();
        remainingSpeedY = playerBall.getSpeedY();

        while (remainingSpeedX != 0 || remainingSpeedY != 0)
        {
            // DEBUGGING
            if (MovementException())
            {
                System.out.println("MOVEMENT EXCEPTION - Should not happen");
            }

            if (HandleBlockBallCollision())
                result = RESULT_BALLMOVEMENT.BLOCK_HIT;

            /* --- Wall Collision checks --- */
            if (BallWillHitLeftWall())
            {
                // ball will hit left wall
                doubleTmp = -remainingSpeedX - playerBall.getLeftSideX();
                ratio = doubleTmp / remainingSpeedX;

                playerBall.setPosX(Globals.BALL_RADIUS_STD);
                remainingSpeedX -= (-remainingSpeedX + doubleTmp);
                remainingSpeedX *= -1;
                playerBall.setSpeedX(-1 * playerBall.getSpeedX());

                playerBall.incPosY((1-ratio) * remainingSpeedY);
                remainingSpeedY -= ((1-ratio) * remainingSpeedY);
            }
            else if (BallWillHitRightWall())
            {
                // ball will hit right wall
                doubleTmp = remainingSpeedX - (Globals.GAMEPANEL_WIDTH - playerBall.getRightSideX());
                ratio = doubleTmp / remainingSpeedX;

                playerBall.setPosX(Globals.GAMEPANEL_WIDTH - Globals.BALL_RADIUS_STD);
                remainingSpeedX -= (remainingSpeedX - doubleTmp);
                remainingSpeedX *= -1;
                playerBall.setSpeedX(-1 * playerBall.getSpeedX());

                playerBall.incPosY((1-ratio) * remainingSpeedY);
                remainingSpeedY -= ((1-ratio) * remainingSpeedY);
            }
            else if (BallWillHitTopWall())
            {
                // ball will hit top wall
                doubleTmp = -remainingSpeedY - playerBall.getTopSideY();
                ratio = doubleTmp / remainingSpeedY;

                playerBall.setPosY(Globals.BALL_RADIUS_STD);
                remainingSpeedY -= (remainingSpeedY + doubleTmp);
                remainingSpeedY *= -1;
                playerBall.setSpeedY(-1 * playerBall.getSpeedY());

                playerBall.incPosX((1-ratio) * remainingSpeedX);
                remainingSpeedX -= ((1-ratio) * remainingSpeedX);
            }

            /* --- Paddle collision check --- */
            else if ( BallWillHitPaddle() )
            {
                // if we come to this point, it means during the next ball movement it hits the paddle

                if ( BallWillHitLeftSideOfPaddle() )
                {
                    // ball hits the left side of the paddle
                    doubleTmp = remainingSpeedX - (playerPaddle.getLeftSideX() - playerBall.getRightSideX());
                    ratio = doubleTmp / remainingSpeedX;

                    playerBall.setPosX(playerPaddle.getLeftSideX() - Globals.BALL_RADIUS_STD);
                    remainingSpeedX -= (remainingSpeedX - doubleTmp);
                    remainingSpeedX *= -1;
                    playerBall.setSpeedX(-1 * playerBall.getSpeedX());

                    playerBall.incPosY((1-ratio) * remainingSpeedY);
                    remainingSpeedY -= ((1-ratio) * remainingSpeedY);
                }

                else if ( BallWillHitRightSideOfPaddle() )
                {
                    // ball hits the right side of the paddle
                    doubleTmp = -remainingSpeedX - (playerPaddle.getRightSideX() - playerBall.getLeftSideX());
                    ratio = doubleTmp / remainingSpeedX;

                    playerBall.setPosX(playerPaddle.getRightSideX() + Globals.BALL_RADIUS_STD);
                    remainingSpeedX -= (-remainingSpeedX - doubleTmp);
                    remainingSpeedX *= -1;
                    playerBall.setSpeedX(-1 * playerBall.getSpeedX());

                    playerBall.incPosY((1-ratio) * remainingSpeedY);
                    remainingSpeedY -= ((1-ratio) * remainingSpeedY);
                }
                else
                {
                    // ball hits top of paddle because ball cannot hit bottom side
                    doubleTmp = playerBall.getSpeedY() - (playerPaddle.getTopSideY() - (playerBall.getPosY() + Globals.BALL_RADIUS_STD));
                    playerBall.setPosY(playerPaddle.getTopSideY() - Globals.BALL_RADIUS_STD - doubleTmp);

                    // calculate bounce angle - if ball hits the middle of paddle, angle is 0
                    doubleTmp = playerBall.getPosX() - playerPaddle.getPosX();
                    // TO DO: check next line!
                    angle = Math.min( (90f / playerPaddle.getWidth()) * doubleTmp - 45, 75);

                    //angle =

                    // calculate old energy
                    v = Math.sqrt((playerBall.getSpeedX() * playerBall.getSpeedX()) + (playerBall.getSpeedY() * playerBall.getSpeedY()));
                    //System.out.print("old: " + v);
                    angle = Math.toRadians(angle);
                    playerBall.setSpeedY(Math.cos(angle) * -v);
                    playerBall.setSpeedX(Math.sin(angle) * v);
                    //System.out.println("new: "+ Math.sqrt((playerBall.getSpeedX() * playerBall.getSpeedX()) + (playerBall.getSpeedY() * playerBall.getSpeedY())));

                    remainingSpeedX = remainingSpeedY = 0;
                }

            }

            /* --- Ball Lost Check --- */
            else if (playerBall.getPosY() >= Globals.GAMEPANEL_HEIGHT)
            {
                result = RESULT_BALLMOVEMENT.BALL_LOST;
                remainingSpeedX = remainingSpeedY = 0;
            }

            /* -- normal movement without collisions --- */
            else
            {
                playerBall.incPosX(remainingSpeedX);
                playerBall.incPosY(remainingSpeedY);
                remainingSpeedX = remainingSpeedY = 0;
            }

            // map very small floatinhg point to zero to assure exiting while-loop
            if (Math.abs(remainingSpeedX) < Globals.EPSILON )
                remainingSpeedX = 0;
            if (Math.abs(remainingSpeedY) < Globals.EPSILON )
                remainingSpeedY = 0;
        } // end while

        return result;
    }


    /*
     * Checks and handles the collision between the ball and a block.
     */
    private boolean HandleBlockBallCollision()
    {
        double doubleTmpRemain;
        int xTmp, yTmp;
        boolean result = false;

        // first check if ball is in the area where the blocks reside
        if (playerBall.getBottomSideY() + remainingSpeedY >= Globals.BLOCK_AREA_TOP &&
            playerBall.getTopSideY() + remainingSpeedY <= Globals.BLOCK_AREA_BOTTOM)
        {
            for (int y = 0; y < Globals.NUM_BLOCKS_Y; y++)
            {
                yTmp = Globals.BLOCK_AREA_TOP + y * Globals.BLOCKSIZE_Y;
                for (int x = 0; x < Globals.NUM_BLOCKS_X; x++)
                {
                    if (!curlevel.IsBlockEmpty(x, y))
                    {
                        xTmp = x * Globals.BLOCKSIZE_X;

                        if ( (playerBall.getBottomSideY() + remainingSpeedY >= yTmp) &&
                             (playerBall.getTopSideY() + remainingSpeedY <= yTmp + Globals.BLOCKSIZE_Y) &&
                             (playerBall.getRightSideX() + remainingSpeedX >= xTmp) &&
                             (playerBall.getLeftSideX() + remainingSpeedX <= xTmp + Globals.BLOCKSIZE_X) )
                        {
                            int blockLeftX = x * Globals.BLOCKSIZE_X;
                            int blockRightX = blockLeftX + Globals.BLOCKSIZE_X;
                            int blockTopY = y * Globals.BLOCKSIZE_Y + Globals.BLOCK_AREA_TOP;
                            int blockBottomY = blockTopY + Globals.BLOCKSIZE_Y;

                            boolean bWasHit = false;
                            result = true;
                            // ball hits a block!
                            if ( (playerBall.getRightSideX() < blockLeftX) &&
                                 (playerBall.getRightSideX() + remainingSpeedX >= blockLeftX) )
                            {
                                // ball will hit left side of the block
                                doubleTmpRemain = remainingSpeedX - (blockLeftX - playerBall.getRightSideX());

                                playerBall.setPosX(blockLeftX - Globals.BALL_RADIUS_STD);
                                remainingSpeedX -= (remainingSpeedX - doubleTmpRemain);
                                remainingSpeedX *= -1;
                                playerBall.setSpeedX(-1 * playerBall.getSpeedX());

                                curlevel.BlockIsHit(x, y);
                                bWasHit = true;
                            }

                            if ( (playerBall.getLeftSideX() > blockRightX) &&
                                 (playerBall.getLeftSideX() + remainingSpeedX <= blockRightX) )
                            {
                                // ball will hit right side of the block
                                doubleTmpRemain = -remainingSpeedX - (blockRightX - playerBall.getLeftSideX());

                                playerBall.setPosX(blockRightX + Globals.BALL_RADIUS_STD);
                                remainingSpeedX -= (-remainingSpeedX + doubleTmpRemain);
                                remainingSpeedX *= -1;
                                playerBall.setSpeedX(-1 * playerBall.getSpeedX());

                                curlevel.BlockIsHit(x, y);
                                bWasHit = true;
                            }
                            if ( (playerBall.getTopSideY() > blockBottomY) &&
                                 (playerBall.getTopSideY() + remainingSpeedY <= blockBottomY) )
                            {
                                // ball will hit bottom side of block
                                doubleTmpRemain = -remainingSpeedY - (blockBottomY - playerBall.getTopSideY());

                                playerBall.setPosY(blockBottomY + Globals.BALL_RADIUS_STD);
                                remainingSpeedY -= (-remainingSpeedY + doubleTmpRemain);
                                remainingSpeedY *= -1;
                                playerBall.setSpeedY(-1 * playerBall.getSpeedY());

                                curlevel.BlockIsHit(x, y);
                                bWasHit = true;
                            }

                            if ( (playerBall.getBottomSideY() < blockTopY) &&
                                 (playerBall.getBottomSideY() + remainingSpeedY >= blockTopY))
                            {
                                // ball will hit top side of block
                                doubleTmpRemain = remainingSpeedY - (blockTopY - playerBall.getBottomSideY());

                                playerBall.setPosY(blockTopY - Globals.BALL_RADIUS_STD);
                                remainingSpeedY -= (remainingSpeedY - doubleTmpRemain);
                                remainingSpeedY *= -1;
                                playerBall.setSpeedY(-1 * playerBall.getSpeedY());

                                curlevel.BlockIsHit(x, y);
                                bWasHit = true;
                            }

                            if (!bWasHit)
                            {
                                // in this case the ball does not hit an edge of the block
                                // so this means it hits directly an vertice of the block which requires special handling

                                if (distanceTo(playerBall.getPosX(), playerBall.getPosY(),
                                        blockLeftX, blockTopY) < Globals.BALL_RADIUS_STD)
                                {
                                    // ball will hit topleft corner of block
                                    bWasHit = true;
                                    remainingSpeedY *= -1;
                                    playerBall.setSpeedY(-1 * playerBall.getSpeedY());
                                    remainingSpeedX *= -1;
                                    playerBall.setSpeedX(-1 * playerBall.getSpeedX());
                                    curlevel.BlockIsHit(x, y);
                                }

                                if (distanceTo(playerBall.getPosX(), playerBall.getPosY(),
                                        blockLeftX, blockTopY + Globals.BLOCKSIZE_Y) < Globals.BALL_RADIUS_STD)
                                {
                                    // ball will hit topright corner of block
                                    bWasHit = true;
                                    remainingSpeedY *= -1;
                                    playerBall.setSpeedY(-1 * playerBall.getSpeedY());
                                    remainingSpeedX *= -1;
                                    playerBall.setSpeedX(-1 * playerBall.getSpeedX());
                                    curlevel.BlockIsHit(x, y);
                                }

                                if (distanceTo(playerBall.getPosX(), playerBall.getPosY(),
                                        blockLeftX + Globals.BLOCKSIZE_X, blockTopY) < Globals.BALL_RADIUS_STD)
                                {
                                    // ball will hit bottomleft corner of block
                                    bWasHit = true;
                                    remainingSpeedY *= -1;
                                    playerBall.setSpeedY(-1 * playerBall.getSpeedY());
                                    remainingSpeedX *= -1;
                                    playerBall.setSpeedX(-1 * playerBall.getSpeedX());
                                    curlevel.BlockIsHit(x, y);
                                }

                                if (distanceTo(playerBall.getPosX(), playerBall.getPosY(),
                                        blockLeftX + Globals.BLOCKSIZE_X, blockTopY + Globals.BLOCKSIZE_Y) < Globals.BALL_RADIUS_STD)
                                {
                                    // ball will hit bottomright corner of block
                                    bWasHit = true;
                                    remainingSpeedY *= -1;
                                    playerBall.setSpeedY(-1 * playerBall.getSpeedY());
                                    remainingSpeedX *= -1;
                                    playerBall.setSpeedX(-1 * playerBall.getSpeedX());
                                    curlevel.BlockIsHit(x, y);
                                }

                                if (!bWasHit)
                                {
                                    System.out.println("This should never happen");
                                    /*while(true)
                                        ;
                                     */
                                     
                                }

                            }
                        } // playerball hits block
                    } // !BlockEmpty
                } // x for-loop
            } // y for-loop
        } // end block area check
        return result;
    }

    /**
     * Calculates the euclidean distance between two points.
     * @param x1 x-coordinate of point 1
     * @param y1 y-coordinate of point 1
     * @param x2 x-coordinate of point 2
     * @param y2 y-coordinate of point 2
     * @return
     */
    private double distanceTo(double x1, double y1, double x2, double y2)
    {
        return Math.sqrt( ((x2-x1)*(x2-x1)) + ((y2-y1)*(y2-y1)) );
    }


    /**
     * Internal debugging function. Checks if there is an internal error in the ball
     * movement physiscs.
     * @return true if the ball 'hangs inside' the paddle or inside a block.
     */
    private boolean MovementException()
    {
        int xTmp, yTmp;
        boolean result = false;

        if ( (playerBall.getBottomSideY() > playerPaddle.getPosY()) &&
             (playerBall.getTopSideY() < playerPaddle.getPosY() + Globals.PADDLE_HEIGHT) &&
             (playerBall.getRightSideX() > playerPaddle.getPosX()) &&
             (playerBall.getLeftSideX() < playerPaddle.getPosX() + playerPaddle.getWidth()))
        {
            result = true;
        }

        for (int y = 0; y < Globals.NUM_BLOCKS_Y; y++)
        {
            yTmp = Globals.BLOCK_AREA_TOP + y * Globals.BLOCKSIZE_Y;
            for (int x = 0; x < Globals.NUM_BLOCKS_X; x++)
            {
                if (!curlevel.IsBlockEmpty(x, y))
                {
                    xTmp = x * Globals.BLOCKSIZE_X;

                    if ( (playerBall.getBottomSideY() > yTmp) &&
                         (playerBall.getTopSideY() < yTmp + Globals.BLOCKSIZE_Y) &&
                         (playerBall.getRightSideX() > xTmp) &&
                         (playerBall.getLeftSideX() < xTmp + Globals.BLOCKSIZE_X) )
                    {
                        result = true;
                    }
                }
            }
        }

        return result;
    }

}
