/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package StoneBreak.StoneBreakGame;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.Logger;

/**
 *
 * @author Sunshine2k
 */
public class GameEngine {

    private static final int DURATION_COUNTDOWN_INTERVALL = 800;

    private static int NumOfLives = 3;



    private enum GameState { STATE_UNINT,
                             STATE_INITIALIZING,    // during loading / initializing
                             STATE_COUNTDOWN,       // countdown is running before current level starts
                             STATE_BALLONPADDLE,    // countdown finished but user did not pressed mouse to start the ball
                             STATE_RUNNING,         // the game is currently running, the user is playing
                             STATE_LEVELDONE,       // level finished, intermediate state to next level switch
                             STATE_GAMEOVER,        // gameover, if user clicks button game restarts from beginning
                             STATE_PAUSED
                            };
    private GameState       state;
    private GameState       oldState;

    private Ball            playerBall;
    private Paddle          playerPaddle;

    private LevelManager    levelManager;

    private GameBallPhysicsInterface  gameLogic;
    
    /**
     * Direction and amount of paddle movement in pixels processed in next tick.
     * Negative value means movement to left, positive to right.
     */
    private int             playerPaddleMovement;

    /**
     * number of lives left
     */
    private int             numOfLivesLeft;

    /* Countdown related variables */
    private int             countdownValue;
    private Timer           countdownTimer;
    private Font            countdownFont;

    /* Gameover related variables */
    private static String           STR_GAMEOVER    = "Game over!";
    private final static String     STR_RESTART     = "Press Mouse to restart!";
    private Font                    gameOverFont;

    /* 'Level finished' state related variables */
    private final static String     STR_LEVELDONE   = "Level finished!";
    private Timer                   levelDoneTimer;

    /* small font for drawing information */
    private Font                infoFont;

    /**
     * Default constructor.
     */
    public GameEngine()
    {
        state = GameState.STATE_UNINT;
    }

    /**
     * Initialization functions - has to be called as first function before any other, starts the game
     */
    public void Init()
    {
        state = GameState.STATE_INITIALIZING;
        numOfLivesLeft = NumOfLives;

        playerBall = new Ball();
        RespawnBall();

        playerPaddle = Paddle.getPaddleInstance();
        playerPaddleMovement = 0;

        levelManager = LevelManager.getLevelManager();
        try {
            levelManager.InitLoadLevels();
        } catch (IOException ex) {
            Logger.getLogger(GameEngine.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }

        gameLogic = new GameBallPhysicsSimple(playerBall, playerPaddle);

        countdownFont = new Font(Font.DIALOG, Font.BOLD, Globals.GAMEPANEL_HEIGHT / 4);
        gameOverFont = new Font(Font.DIALOG, Font.BOLD, Globals.GAMEPANEL_HEIGHT / 10);
        infoFont = new Font(Font.DIALOG, Font.BOLD, Globals.GAMEPANEL_HEIGHT / 35);
        StartStateCountDown();
       
    }

    /**
     * Helper function to reset position and speed of the playerball to initial state.
     */
    private void RespawnBall()
    {
        playerBall.setPosX(Globals.BALL_INITPOS_X);
        playerBall.setPosY(Globals.BALL_INITPOS_Y);
        playerBall.setSpeedX(0);
        playerBall.setSpeedY(-Globals.BALL_TOTALSPEED);
    }

    /**
     * Main loop of the game where all the processing (graphics, interaction, logic) goes.
     * Must be called with a fixed timebase.
     * @param g Graphics object of the panel to draw on
     */
    public void MainLoop(Graphics2D g)
    {
        switch (state)
        {
            case STATE_RUNNING:
            {
                ProcessGame();
                DrawGame(g);
            }
            break;

            case STATE_COUNTDOWN:
            {
                ProcessStateStartCountDown(g);
            }
            break;

            case STATE_GAMEOVER:
            {
                ProcessStateGameOver(g);
            }
            break;

            case STATE_LEVELDONE:
            {
                DrawGame(g);
                ProcessStateLevelDone(g);
            }
            break;

            case STATE_BALLONPADDLE:
            {
                // in this state the level is visible and the paddle is movable
                // but the ball is not moving yet...
                DrawGame(g);
                ProcessPaddle();
                // move the ball with the paddle
                playerBall.setPosX(playerPaddle.getPosX() + (playerPaddle.getWidth() / 2));
            }
            break;

            case STATE_PAUSED:
            {
                DrawGame(g);
            }
            break;
            
            default: break;     /* should never happen */
        }
    }

    /**
     * Process the game in state game over.
     * This state can only be left if user clicks a mouse button.
     * @param g Graphics object to be drawn on
     */
    private void ProcessStateGameOver(Graphics2D g)
    {
        Font oldFont;
        Color oldColor;
        
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        oldColor = g.getColor();
        oldFont = g.getFont();
        g.setColor(Color.LIGHT_GRAY);
        g.setFont(gameOverFont);

        FontMetrics fm = g.getFontMetrics();
        g.drawString(STR_GAMEOVER, (Globals.GAMEPANEL_WIDTH - fm.stringWidth(STR_GAMEOVER)) >> 1,
                (fm.getAscent() + (Globals.GAMEPANEL_HEIGHT - (fm.getAscent() + fm.getDescent() + fm.getHeight())) / 2));
        g.drawString(STR_RESTART, (Globals.GAMEPANEL_WIDTH - fm.stringWidth(STR_RESTART)) >> 1,
                (fm.getAscent() + (Globals.GAMEPANEL_HEIGHT - (fm.getAscent() + fm.getDescent() - fm.getHeight())) / 2));

        g.setFont(oldFont);
        g.setColor(oldColor);
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
    }

    /**
     * Process the game in stat game over.
     * @param g Graphics object to be drawn on
     */
    private void ProcessStateLevelDone(Graphics2D g)
    {
        Font oldFont;
        Color oldColor;

        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        oldColor = g.getColor();
        oldFont = g.getFont();
        g.setColor(Color.LIGHT_GRAY);
        g.setFont(gameOverFont);

        FontMetrics fm = g.getFontMetrics();
        g.drawString(STR_LEVELDONE, (Globals.GAMEPANEL_WIDTH - fm.stringWidth(STR_LEVELDONE)) >> 1,
                (fm.getAscent() + (Globals.GAMEPANEL_HEIGHT - (fm.getAscent() + fm.getDescent())) / 2));

        g.setFont(oldFont);
        g.setColor(oldColor);
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
    }

    /**
     * 
     * @param g
     * @param font
     * @param strToPrint
     */
    private void DrawScoreString(Graphics2D g, Font font, String strToPrint)
    {
        Font oldFont;
        Color oldColor;
        
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        oldColor = g.getColor();
        oldFont = g.getFont();
        g.setColor(Color.LIGHT_GRAY);
        g.setFont(font);

        FontMetrics fm = g.getFontMetrics();
        g.drawString(strToPrint, Globals.GAMEPANEL_WIDTH - fm.stringWidth(strToPrint) - 5,
                Globals.GAMEPANEL_HEIGHT - 5);

        g.setFont(oldFont);
        g.setColor(oldColor);
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
    }

    /**
     * All logic/input/movements of the game is processed here.
     */
    private void ProcessGame()
    {
        // process the ball
        GameBallPhysicsInterface.RESULT_BALLMOVEMENT res = gameLogic.ProcessBall(levelManager.getCurLevel());

        if (res == GameBallPhysicsInterface.RESULT_BALLMOVEMENT.BALL_LOST)
        {
            numOfLivesLeft--;

            if (numOfLivesLeft < 0)
            {
                // no lives left -> gameover
                STR_GAMEOVER = "Game over!";
                state = GameState.STATE_GAMEOVER;
            }
            else
            {
                // remove active special item
                playerPaddle.removeSpecialFeatures();
                playerBall.setDefaultColor();

                // delete all visible special items
                GameLevel curlevel = levelManager.getCurLevel();
                for (int y = 0; y < Globals.NUM_BLOCKS_Y; y++)
                {
                    for (int x = 0; x < Globals.NUM_BLOCKS_X; x++)
                    {
                         // move special item
                        curlevel.MoveSpecialItem(x, y);

                        if (curlevel.IsSpecialItemActive(x, y))
                        {
                            curlevel.RemoveSpecialItem(x, y);
                        }
                    }
                }
                
                // start countdown
                StartStateCountDown();
                playerPaddle.setDefaultPosition();
                RespawnBall();
            }
        }

        else if (res == GameBallPhysicsInterface.RESULT_BALLMOVEMENT.BLOCK_HIT)
        {
            // at least one block was hit, check if level is finished
            if (levelManager.IsCurrentLevelFinsihed())
            {
                StartStateLevelDone();
            }
        }
        
        // process the player paddle here...
        ProcessPaddle();
        // process the special items
        ProcessSpecialItems();
    }

    /**
     * Handles the transition to state 'leveldone'. Called if user hits last block and so finishes the level.
     */
    private void StartStateLevelDone()
    {
        state = GameState.STATE_LEVELDONE;
        playerPaddle.removeSpecialFeatures();
        playerBall.setDefaultColor();

        // start timer to switch to new level
        levelDoneTimer = new Timer();

        TimerTask levelDoneTimerTask = new TimerTask() {
            public void run()
            {
                if (levelManager.SwitchToNextLevel())
                {
                    levelManager.ReInitCurrentLevel();
                    playerPaddle.setDefaultPosition();
                    RespawnBall();
                    // start countdown
                    StartStateCountDown();

                }
                else
                {
                    STR_GAMEOVER = "Game DONE! Score: " + levelManager.getTotalScore();
                    state = GameState.STATE_GAMEOVER;
                }
            }
        };

        levelDoneTimer.schedule(levelDoneTimerTask, 4000);
    }
    /*
     * Start the countdown counter before the game starts.
     * Called when new level starts and ball is lost.
     */
    private void StartStateCountDown()
    {
        countdownValue = 3;
        countdownTimer = null;  /* explicitely remove reference for gc */
        countdownTimer = new Timer();

        TimerTask countdownTimerTask = new TimerTask() {
            public void run()
            {
                if (countdownValue <= 1)
                {
                    countdownTimer.cancel();
                    
                    state = GameState.STATE_BALLONPADDLE;
                }
                else
                {
                    countdownValue--;
                }
            }
        };
        countdownTimer.schedule(countdownTimerTask, DURATION_COUNTDOWN_INTERVALL, DURATION_COUNTDOWN_INTERVALL);
        state = GameState.STATE_COUNTDOWN;
    }

    /**
     * Drawing function while in countdown state
     * @param g Graphics object to be drawn on
     */
    private void ProcessStateStartCountDown(Graphics2D g)
    {
        Font oldFont;
        Color oldColor;
        String countDownStr = Integer.toString(countdownValue);
        String levelStr = "Level " + Integer.toString(levelManager.getCurLevelNum() + 1);

        // draw the game normally
        DrawGame(g);

        // draw the countdown value
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        oldColor = g.getColor();
        oldFont = g.getFont();
        g.setColor(Color.LIGHT_GRAY);
        g.setFont(countdownFont);
        FontMetrics fm = g.getFontMetrics();
        g.drawString(levelStr, (Globals.GAMEPANEL_WIDTH - fm.stringWidth(levelStr)) >> 1,
                (fm.getAscent() + (Globals.GAMEPANEL_HEIGHT - (fm.getAscent() + fm.getDescent() + fm.getHeight())) / 2));
        g.drawString(countDownStr, (Globals.GAMEPANEL_WIDTH - fm.stringWidth(countDownStr)) >> 1,
                (fm.getAscent() + (Globals.GAMEPANEL_HEIGHT - (fm.getAscent() + fm.getDescent() - fm.getHeight())) / 2));
        // reset default graphics values
        g.setFont(oldFont);
        g.setColor(oldColor);
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
    }

    /**
     * Handle paddle movement
     */
    private void ProcessPaddle()
    {
        playerPaddle.incPosX(playerPaddleMovement);
        playerPaddleMovement = 0;
        if (playerPaddle.HandleSpecialFeatures(levelManager.getCurLevel()))
        {
            // at least one block was hit, check if level is finished
            if (levelManager.IsCurrentLevelFinsihed())
            {
                StartStateLevelDone();
            }
        }
    }


    /**
     * Handle the logic of special items; move them and handle when player catches it.
     * Note that when a special item is once catched, the processing is done at the corrsponding object.
     */
    private void ProcessSpecialItems()
    {
        GameLevel curlevel = levelManager.getCurLevel();

        for (int y = 0; y < Globals.NUM_BLOCKS_Y; y++)
        {
            for (int x = 0; x < Globals.NUM_BLOCKS_X; x++)
            {
                 // move special item
                curlevel.MoveSpecialItem(x, y);

                if (curlevel.IsSpecialItemActive(x, y))
                {
                    if (PaddleWillHitSpecialItem(curlevel.GetSpecialPosX(x, y),
                                                 curlevel.GetSpecialPosY(x, y)))
                    {
                        // paddle hits special item, so handle it
                        Globals.BLOCK_SPECIAL_ITEM specialItem;

                        // first remove all special items from the object because
                        // only one feature may be active at the same time
                        playerPaddle.removeSpecialFeatures();
                        
                        specialItem = curlevel.GetSpecialItem(x, y);
                        switch (specialItem)
                        {
                            case PADDLE_GROWTH:
                            {
                                playerPaddle.StartGrowth();
                                levelManager.addScore(Globals.SISCORE_GROWTH);
                                playerBall.setDefaultColor();
                            }
                            break;

                            case EXTRA_LIVE:
                            {
                                numOfLivesLeft++;
                                levelManager.addScore(Globals.SISCORE_EXTRALIFE);
                                playerBall.setDefaultColor();
                            }
                            break;

                            case PADDLE_SHOOT:
                            {
                                playerPaddle.AddShootingFeature(Paddle.GUNSHOT_TYPE.NORMAL);
                                levelManager.addScore(Globals.SISCORE_SHOOT);
                                playerBall.setDefaultColor();
                            }
                            break;

                            case PADDLE_SHOOTEX:
                            {
                                playerPaddle.AddShootingFeature(Paddle.GUNSHOT_TYPE.EXTENDED);
                                levelManager.addScore(Globals.SISCORE_SHOOTEX);
                                playerBall.setDefaultColor();
                            }
                            break;

                            case BALL_HIT_NEIGHBOURS:
                            {
                                playerPaddle.NeighbourHitFeatureActive = true;
                                playerBall.setBallColor(Globals.SI_NEIGHBOURBLOCKSHIT_BALL_COLOR);
                                levelManager.addScore(Globals.SISCORE_NEIGHBOURBLOCK);
                            }
                            break;
                            
                            default: break;
                        }
                        curlevel.RemoveSpecialItem(x, y);
                    }
                }
            }
        }
    }

    /**
     * Check if paddle hits a falling special item
     * @param six x-coordinate of special item
     * @param siy y-coordinate of special item
     * @return true if paddle hits a special item, otherwise false.
     */
    private boolean PaddleWillHitSpecialItem(double six, double siy)
    {
        return ( (six + Globals.SPECIAL_ITEM_SIZEX > playerPaddle.getPosX()) &&
                 (six < (playerPaddle.getPosX() + playerPaddle.getWidth())) &&
                 (siy + Globals.PADDLE_HEIGHT + Globals.SPECIAL_ITEM_SPEEDY >= playerPaddle.getPosY()) &&
                 (siy <= playerPaddle.getPosY() + Globals.PADDLE_HEIGHT) );
                 
    }


    /**
     * Main function to draw the game
     * @param g Graphics object to be drawn on
     */
    private void DrawGame(Graphics2D g)
    {
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        
        playerPaddle.paint(g);
        levelManager.DrawCurrentLevel(g);
        playerBall.paint(g);

        // draw the remaining lives
        for (int i = 0; i < numOfLivesLeft; i++)
        {
            Ball.DrawBall(g, 10 + i * 15, Globals.GAMEPANEL_HEIGHT - 10, 6);
        }

        // draw current score
        DrawScoreString(g, infoFont, "Score: " + levelManager.getTotalScore());

        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
    }


    /**
     * Function is meant to be used for keyboard handling of the player paddle.
     * Call this each time a key is pressed.
     * @param dir direction of paddle movement
     */
    public void PlayerPaddleMoveOneStep(Paddle.MOVE_DIRECTION dir)
    {
        if (state == GameState.STATE_RUNNING || state == GameState.STATE_BALLONPADDLE)
        {   // handle player movements only on running game
            if (dir == Paddle.MOVE_DIRECTION.RIGHT)
            {
                playerPaddleMovement += Globals.PADDLE_MOVEDISTANCE_PER_KEYPRESS;
            }
            else if (dir == Paddle.MOVE_DIRECTION.LEFT)
            {
                playerPaddleMovement -= Globals.PADDLE_MOVEDISTANCE_PER_KEYPRESS;
            }
        }
    }

    /**
     * Set new x-position of the player paddle. Function is useful for mouse
     * handling of the player paddle.
     * @param xPos new horizontal position of paddle
     */
    public void PlayerPaddleNewPosition(int xPos)
    {
        if (state == GameState.STATE_RUNNING || state == GameState.STATE_BALLONPADDLE)
        {   // handle player movements only on running game
            playerPaddleMovement = xPos - playerPaddle.getPosX();
        }
    }

    /**
     * Called when user presses a mouse button.
     * @param me struct with information about this event
     */
    public void HandleMouseClick(MouseEvent me)
    {
        if (me.getButton() == MouseEvent.BUTTON1)
        {
            if (state == GameState.STATE_GAMEOVER)
            {
                levelManager.SwitchToLevel(0);
                levelManager.ReInitLevel(0);
                levelManager.resetTotalScore();
                numOfLivesLeft = NumOfLives;

                // start countdown
                StartStateCountDown();
                playerPaddle.setDefaultPosition();
                RespawnBall();
            }
            else if (state == GameState.STATE_BALLONPADDLE)
            {
                state = GameState.STATE_RUNNING;
            }
            else if (state == GameState.STATE_LEVELDONE)
            {
                // we can quit the 'level finished' screen with a mousepress
                levelDoneTimer.cancel();
                levelDoneTimer = null;
                if (levelManager.SwitchToNextLevel())
                {   // level finished and there is still a level left -> go to next level
                    levelManager.ReInitCurrentLevel();
                    // start countdown
                    StartStateCountDown();
                    playerPaddle.setDefaultPosition();
                    RespawnBall();
                }
                else
                {
                    // no level remains -> game finished, same as gameover state but
                    // another message ;-)
                    STR_GAMEOVER = "Game DONE! Score: " + levelManager.getTotalScore();
                    state = GameState.STATE_GAMEOVER;
                }
            }
            else if (state == GameState.STATE_RUNNING)
            {
                // in case the playerpaddle owns currently the shooting feature
                // mouseclicks must be forwarded
                playerPaddle.TriggerShot();
            }
        }
        else if (me.getButton() == MouseEvent.BUTTON3)
        {
            // right mouse button clicked - pauses or resumes game
            if (state == GameState.STATE_RUNNING)
            {
                // pause game
                oldState = state;
                state = GameState.STATE_PAUSED;
            }
            else if (state == GameState.STATE_PAUSED)
            {
                // resume game
                state = oldState;
            }
        }
    }
    

    /* ---------------------------------------------------------------------- */
    /* Internal test functions                                                */
    /* ---------------------------------------------------------------------- */
    public void decBallSpeed()
    {
        playerBall.incSpeedX(0.9);
        playerBall.incSpeedY(0.9);
    }
    public void incBallSpeed()
    {
        playerBall.incSpeedX(1.1);
        playerBall.incSpeedY(1.1);
    }

    public void beamToNextLevel()
    {
         StartStateLevelDone();
    }
    
}
