/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package StoneBreak.StoneBreakGame;

import java.awt.Color;

/**
 *
 * @author Sunshine2k
 */
public class Globals {
    
    /* ----------------------------------------------------------------------
     * General variables
     * ---------------------------------------------------------------------- */
    /**
     * width of applet in pixels
     */
    public static int APPLET_WIDTH      = 700; //700;
    /**
     * height of applet in pixels
     */
    public static int APPLET_HEIGHT     = (APPLET_WIDTH * 3) / 4; //500;
    /**
     * width of actual drawing panel in pixels
     */
    public static int GAMEPANEL_WIDTH   = APPLET_WIDTH;
    /**
     * height of actual drawing panel in pixels
     */
    public static int GAMEPANEL_HEIGHT  = APPLET_HEIGHT;
    /**
     * target frames per second to achieve - influences game speed
     */
    public static final int FPS_TARGET        = 80;

    /**
     * maximal number of blocks in a row = number of columns
     */
    public static final int NUM_BLOCKS_X       = 20;
    /**
     * maximal number of blocks in a column = number of rows
     */
    public static final int NUM_BLOCKS_Y       = 15;


    /**
     * the free area at the top where no blocks are allowed, in percentage
     */
    public static final double AREA_BLOCKS_Y_BEGIN_PERCENTAGE   = 0.1;
    /**
     * the end of the block area from the top, in percentage
     */
    public static final double AREA_BLOCKS_Y_END_PERCENTAGE   = 0.65;

    /**
     * the epsilon value used for checking floating point variables of zero
     */
    public static final double EPSILON = 0.01;

    /* ----------------------------------------------------------------------
     * Block related variables
     * ---------------------------------------------------------------------- */
    /**
     * y-position in pixels from the top of the gamepanel where the area with the blocks begin
     */
    public static int BLOCK_AREA_TOP = (int)(GAMEPANEL_HEIGHT * AREA_BLOCKS_Y_BEGIN_PERCENTAGE);
    /**
     * size of one block in x-direction
     */
    public static int BLOCKSIZE_X = GAMEPANEL_WIDTH / NUM_BLOCKS_X;
    /**
     * size of one block in y-direction
     */
    public static int BLOCKSIZE_Y = (int)((GAMEPANEL_HEIGHT * (AREA_BLOCKS_Y_END_PERCENTAGE - AREA_BLOCKS_Y_BEGIN_PERCENTAGE)) / NUM_BLOCKS_Y);
     /**
     * y-position in pixels from the top of the gamepanel where the area with the blocks begin
     */
    public static int BLOCK_AREA_BOTTOM = (int)(GAMEPANEL_HEIGHT * AREA_BLOCKS_Y_BEGIN_PERCENTAGE + NUM_BLOCKS_Y * BLOCKSIZE_Y);

    /* Block Score Values */
    public static final int BLOCKSCORE_TYPE_EMPTY               = 0;
    public static final int BLOCKSCORE_TYPE_2HITS               = 10;
    public static final int BLOCKSCORE_TYPE_3HITS               = 20;
    public static final int BLOCKSCORE_TYPE_NORMALGRADIENT      = 8;
    public static final int BLOCKSCORE_TYPE_STANDARD            = 5;
    public static final int BLOCKSCORE_TYPE_UNBREAKABLE         = 0;

    /* Special item score values */
    public static final int SISCORE_GROWTH                      = 5;
    public static final int SISCORE_EXTRALIFE                   = 15;
    public static final int SISCORE_SHOOT                       = 20;
    public static final int SISCORE_SHOOTEX                     = 25;
    public static final int SISCORE_NEIGHBOURBLOCK              = 20;


    /* ----------------------------------------------------------------------
     * Paddle related variables
     * ---------------------------------------------------------------------- */

    // initial values for the player paddle
    /**
     * Initial width of paddle
     */
    public static int PADDLE_WIDTH  = (80 * APPLET_WIDTH) / 700;
    /**
     * Initial height of paddle
     */
    public static int PADDLE_HEIGHT = (6 * APPLET_WIDTH) / 700;
    /**
     * Default y-position of the paddle in pixels from the top
     */
    public static int PADDLE_DEFAULT_POS_Y = (484 * APPLET_WIDTH / 700);   //460;

    /*
     * Colors used to draw the gradient of the paddle
     */
    public static final Color PADDLE_COLOR_DARK = new Color(92, 94, 109);
    public static final Color PADDLE_COLOR_LIGHT = new Color(160, 162, 171);

    /**
     * Movement of paddle in pixels for one keypress
     */
    public static int PADDLE_MOVEDISTANCE_PER_KEYPRESS = (10 * APPLET_WIDTH / 700);


    /* ------------- Paddle Growth Animation ------------- */

    public static final double PADDLE_GROWTH_PERCENTAGE = 1.4;

    public static final Color PADDLE_SPECIAL_COLOR_DARK = new Color(21, 120, 20);
    public static final Color PADDLE_SPECIAL_COLOR_LIGHT = new Color(0, 225, 0);

    /* ----------------------------------------------------------------------
     * Ball related variables
     * ---------------------------------------------------------------------- */
    /**
     * intial radius of the ball
     */
    public static int BALL_RADIUS_STD     = (6 * APPLET_WIDTH / 700);   //6;
    /**
     * standard color of the ball
     */
    public static final Color BALL_COLOR  = Color.GRAY;

    public static int BALL_INITPOS_X      = (Globals.GAMEPANEL_WIDTH) >> 1;
    public static int BALL_INITPOS_Y      = Globals.PADDLE_DEFAULT_POS_Y - Globals.BALL_RADIUS_STD;

    /**
     * Total speed amount of ball - remains constant
     */
    public static double BALL_TOTALSPEED       = (3.5 * APPLET_WIDTH / 700);


    /* ----------------------------------------------------------------------
     * Block Animation related variables
     * ---------------------------------------------------------------------- */

    public static int BLOCK_NUM_PARTICLES = 70;



    /* ----------------------------------------------------------------------
     * Special item related variables
     * ---------------------------------------------------------------------- */

    /**
     * These are all available special featurs
     */
    public static enum BLOCK_SPECIAL_ITEM {
        NONE,               /* does not contain any special item */
        PADDLE_GROWTH,      /* paddle width will increase */
        EXTRA_LIVE,         /* player receives an extra life */
        PADDLE_SHOOT,       /* paddle is able to shoot */
        PADDLE_SHOOTEX,     /* paddle is able to shoot, shot does not disappear when hit a block */
        BALL_HIT_NEIGHBOURS /* all four neightboring blocks also explode */
        };


    /**
     * drop speed of special item
     */
    public static int SPECIAL_ITEM_SPEEDY = (2 * APPLET_WIDTH / 700);
    /**
     * size of diameter arc of the corner
     */
    public static int SPECIAL_ITEM_ROUNDRECT = (12  * APPLET_WIDTH / 700);
    /**
     * width of paddle growth special item
     */
    public static int SPECIAL_ITEM_SIZEX = (26 * APPLET_WIDTH / 700);

    /**
     * height of paddle growth special item
     */
    public static int SPECIAL_ITEM_SIZEY = (12 * APPLET_WIDTH / 700);


    /* ---------- SPECIAL ITEM: PADDLE GROWTH ---------- */
    /**
     * color of growth special item
     */
    public static final Color SI_PADDLEGROWTH_COLOR = Color.GREEN;

    /**
     * size of the small 'paddle' symbol painted on the special item
     */
    public static int SI_PADDLEGROWTH_WIDTH = SPECIAL_ITEM_SIZEX >> 1;
    public static int SI_PADDLEGROWTH_HEIGHT = SPECIAL_ITEM_SIZEY >> 1;


    /* ---------- SPECIAL ITEM: EXTRA LIFE ---------- */
    /**
     * radius of the small 'ball' symbol painted on the special item
     */
    public static int SI_EXTRALIFE_BALLRADIUS = 
            ( (int)((SPECIAL_ITEM_SIZEY>>1)*0.75) * APPLET_WIDTH / 700);


    /* ---------- SEPCIAL ITEM: SHOOTING PADDLE ---------- */
    /**
     * color of the special item
     */
    public static final Color SI_PADDLESHOOT_COLOR = new Color(255, 220, 220);
    /**
     * width of a single gunshot
     */
    public static int SI_PADDLESHOT_WIDTH = ((PADDLE_WIDTH / 10) * APPLET_WIDTH / 700);
    /**
     * height of a single gunshot
     */
    public static int SI_PADDLESHOT_HEIGHT = PADDLE_HEIGHT << 1;
    /**
     * default speed of a gunshot
     */
    public static double SI_PADDLESHOT_SPEED = (5  * APPLET_WIDTH / 700);
    
    
    /* ---------- SEPCIAL ITEM: SHOOTING PADDLE EXTENDED ---------- */
    /**
     * color of the special item
     */
    public static final Color SI_PADDLESHOOTEX_COLOR = new Color(255, 100, 100);
    /**
     * height of a single extended gunshot
     */
    public static int SI_PADDLESHOTEX_HEIGHT = PADDLE_HEIGHT << 3;
    /**
     * width of a single extended gunshot
     */
    public static int SI_PADDLESHOTEX_WIDTH = PADDLE_WIDTH / 15;

    /* ---------- SEPCIAL ITEM: NEIGHBORING BLOCKS EXPLODE ---------- */
    /**
     * color of the special item
     */
    public static final Color SI_NEIGHBOURBLOCKSHIT_COLOR = new Color(155, 200, 200);
    /**
     * color of ball if special item is active
     */
    public static final Color SI_NEIGHBOURBLOCKSHIT_BALL_COLOR       = new Color(164, 64, 64);


    /**
     * Calculate all variable values depending on the given width of the gamepanel.
     * @param w new width of gamepanel
     */
    public static void CalculateAllVariables(int gameWidth)
    {
        APPLET_WIDTH      = gameWidth;
        APPLET_HEIGHT     = (APPLET_WIDTH * 3) / 4;
        GAMEPANEL_WIDTH   = APPLET_WIDTH;
        GAMEPANEL_HEIGHT  = APPLET_HEIGHT;

        BLOCK_AREA_TOP = (int)(GAMEPANEL_HEIGHT * AREA_BLOCKS_Y_BEGIN_PERCENTAGE);
        BLOCKSIZE_X    = GAMEPANEL_WIDTH / NUM_BLOCKS_X;
        BLOCKSIZE_Y    = (int)((GAMEPANEL_HEIGHT * (AREA_BLOCKS_Y_END_PERCENTAGE - AREA_BLOCKS_Y_BEGIN_PERCENTAGE)) / NUM_BLOCKS_Y);
        BLOCK_AREA_BOTTOM = (int)(GAMEPANEL_HEIGHT * AREA_BLOCKS_Y_BEGIN_PERCENTAGE + NUM_BLOCKS_Y * BLOCKSIZE_Y);

        PADDLE_WIDTH  = (80 * APPLET_WIDTH) / 700;
        PADDLE_HEIGHT = (6 * APPLET_WIDTH) / 700;
        PADDLE_DEFAULT_POS_Y             = (484 * APPLET_WIDTH / 700);
        PADDLE_MOVEDISTANCE_PER_KEYPRESS = (10 * APPLET_WIDTH / 700);

        BALL_RADIUS_STD         = (6 * APPLET_WIDTH / 700);
        BALL_INITPOS_X          = (Globals.GAMEPANEL_WIDTH) >> 1;
        BALL_INITPOS_Y          = Globals.PADDLE_DEFAULT_POS_Y - Globals.BALL_RADIUS_STD;
        BALL_TOTALSPEED         = (3.5 * APPLET_WIDTH / 700);

        SPECIAL_ITEM_SPEEDY     = (2 * APPLET_WIDTH / 700);
        SPECIAL_ITEM_ROUNDRECT  = (12  * APPLET_WIDTH / 700);
        SPECIAL_ITEM_SIZEX      = (26 * APPLET_WIDTH / 700);
        SPECIAL_ITEM_SIZEY      = (12 * APPLET_WIDTH / 700);

        SI_PADDLEGROWTH_WIDTH   = SPECIAL_ITEM_SIZEX >> 1;
        SI_PADDLEGROWTH_HEIGHT  = SPECIAL_ITEM_SIZEY >> 1;

        SI_EXTRALIFE_BALLRADIUS = ( (int)((SPECIAL_ITEM_SIZEY>>1)*0.75) * APPLET_WIDTH / 700);

        SI_PADDLESHOT_WIDTH     = ((PADDLE_WIDTH / 10) * APPLET_WIDTH / 700);
        SI_PADDLESHOT_HEIGHT    = PADDLE_HEIGHT << 1;
        SI_PADDLESHOT_SPEED     = (5  * APPLET_WIDTH / 700);

        SI_PADDLESHOTEX_HEIGHT  = PADDLE_HEIGHT << 3;
        SI_PADDLESHOTEX_WIDTH   = PADDLE_WIDTH / 15;
    }
 }
