/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package StoneBreak.StoneBreakGame;

import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Graphics2D;

/**
 *
 * @author Administrator
 */
public class Paddle {

    public static enum MOVE_DIRECTION { RIGHT, LEFT };

    // current position of paddle - topleft corner of paddle
    private int      x, y;
    // current width of panel
    private int      width;

    private Color   curColorLight, curColorDark;

    /* ----------------------------------------------------------------------
     * Paddle Animtation related variables
     * ---------------------------------------------------------------------- */    
    /**
     * type / direction of growing animation
     */
    public static enum PADDLE_ANI { GROWING, SHRINKING };

    /**
     * current type of animation
     */
    private PADDLE_ANI aniCurrent;

    /**
     * helper variable to handle the animation
     */
    private int aniStepsLeft = -1;


    /* ----------------------------------------------------------------------
     * Shooting feature related variables
     * ---------------------------------------------------------------------- */
    private static final int SHOT_LEFT = 0;
    private static final int SHOT_RIGHT = 1;

    /**
     * Hold if paddle has currently the ability to shoot.
     */
    private boolean isShootingFeatureActive = false;

    /**
     * Represents the two bullets of a shot, left and right. Note that only
     * shot can be active at a time.
     */
    private GunShot shots[] = new GunShot[2];

    /**
     * Hold different bullet types.
     */
    public static enum GUNSHOT_TYPE { NORMAL,   /* normal shot which can only hit one block */
                                      EXTENDED, /* extended shot which is destroys all blocks in its way */
    }

    public boolean NeighbourHitFeatureActive = false;
    
    private static Paddle instance = null;

    /**
     * Get singleton instance of paddle.
     * @return levelmanager paddle
     */
    public static Paddle getPaddleInstance()
    {
        if (instance == null)
            instance = new Paddle();
        return instance;
    }

    /**
     * Default constructor.
     */
    private Paddle()
    {
        // build paddle with default position
        this((Globals.GAMEPANEL_WIDTH - Globals.PADDLE_WIDTH) >> 1,
              Globals.PADDLE_DEFAULT_POS_Y );
        // set default color
        curColorLight = Globals.PADDLE_COLOR_LIGHT;
        curColorDark = Globals.PADDLE_COLOR_DARK;
    }

    /**
     * Constructor with given initial position of the paddle.
     */
    private Paddle(int posX, int posY)
    {
        this.x = posX;
        this.y = posY;
        this.width = Globals.PADDLE_WIDTH;

        shots[SHOT_LEFT] = new GunShot();
        shots[SHOT_RIGHT] = new GunShot();
    }

    /**
     * Draw the paddle with some dummy light effect.
     * @param g Graphics2D object where to draw
     */
    public void paint(Graphics2D g)
    {
        Color oldColor = g.getColor();

        // paint the paddle
        GradientPaint gp2 = new GradientPaint(x, y, curColorLight, x, y + (int)(Globals.PADDLE_HEIGHT*0.5),
                curColorDark, true);
        g.setPaint(gp2);    
        g.fillRoundRect(x, y, width, Globals.PADDLE_HEIGHT, 8, 8);

        // paint shots
        if (isShootingFeatureActive && IsAShotVisible())
        {
            shots[SHOT_LEFT].draw(g);
            shots[SHOT_RIGHT].draw(g);
        }
        
        g.setColor(oldColor);
    }

    /**
     * Handle special features (if any) of the paddle. Must be called in each cycle.
     * @param level pointer to current level
     * @return true if one or more blocks were hit; false if no block was hit
     */
    public boolean HandleSpecialFeatures(GameLevel level)
    {
        boolean result = false;
        // handle growing / shrinking of paddle
        if (aniStepsLeft > 0)
        {
            /* animation in progress */
            if (aniCurrent == PADDLE_ANI.GROWING)
            {
                width+=2;
                x -= 1;
            }
            else
            {
                width -= 2;
                x += 1;
            }
            aniStepsLeft--;     // decrease animation steps to do
            // stop animation if necessary
            if (aniStepsLeft == 0)
                aniStepsLeft = -1;
        }
        
        // handle shooting feature
         if (isShootingFeatureActive && IsAShotVisible())
        {
            shots[SHOT_LEFT].y -= Globals.SI_PADDLESHOT_SPEED;
            shots[SHOT_RIGHT].y -= Globals.SI_PADDLESHOT_SPEED;

            // check if shot moves out of the panel
            if (shots[SHOT_LEFT].y < 0) shots[SHOT_LEFT].visible = false;
            if (shots[SHOT_RIGHT].y < 0) shots[SHOT_RIGHT].visible = false;

            // collision handling
            result = HandleGunShotLevelBlockCollision(level);
        }
        return result;
    }

    /**
     * Returns the current x-position of the paddle
     * @return x-coordinate in pixel of the paddle
     */
    public int getPosX() { return this.x; }
    /**
     * Returns the current y-position of the paddle
     * @return y-coordinate in pixel of the paddle
     */
    public int getPosY() { return this.y; }

    /**
     * Increase/Decrease the x-position of the paddle.
     * @param xDelta amount to increase/decrease the x-coordinate in pixel
     */
    public void incPosX(int xDelta)
    {
        this.x += xDelta;
        // clip the paddle againt the walls
        if (this.x < 0)
            this.x = 0;
        if (this.x + this.width > Globals.GAMEPANEL_WIDTH)
            this.x = Globals.GAMEPANEL_WIDTH - this.width;
    }

    /**
     * @return returns the x-position of the left side of the paddle
     */
    public int getLeftSideX() { return this.x; }

    /**
     * @return returns the y-position of the top side of the paddle
     */
    public int getTopSideY() { return this.y; }

    /**
     * @return returns the x-position of the right side of the paddle
     */
    public int getRightSideX() { return (this.x + this.width); }

    /**
     * @return returns the y-position of the bottom side of the paddle
     */
    public int getBottomSideY() { return (this.y + Globals.PADDLE_HEIGHT); }

    /**
     * @return returns current width of the paddle in pixels
     */
    public int getWidth() { return this.width; }

    /**
     * Reset the paddle to default centered position.
     */
    public void setDefaultPosition()
    {
        this.x = (Globals.GAMEPANEL_WIDTH - Globals.PADDLE_WIDTH) >> 1;
        this.y =  Globals.PADDLE_DEFAULT_POS_Y;
    }


    /**
     * Set player paddle back to default, removing all applied special item features
     */
    public void removeSpecialFeatures()
    {
        width = Globals.PADDLE_WIDTH;
        // set default color
        curColorLight = Globals.PADDLE_COLOR_LIGHT;
        curColorDark = Globals.PADDLE_COLOR_DARK;
        // remove shooting
        shots[SHOT_LEFT].visible = false;
        shots[SHOT_RIGHT].visible = false;
        isShootingFeatureActive = false;
        NeighbourHitFeatureActive = false;
    }
    
    /* ----------------------------------------------------------------------
     * Paddle Animtation
     * ---------------------------------------------------------------------- */

    /**
     * Start the growing animation if special item for wider paddle was collected.
     */
    public void StartGrowth()
    {
        if (aniStepsLeft != -1) /* temporary only one growth allowed */
            return;
        
        aniCurrent = PADDLE_ANI.GROWING;
        aniStepsLeft = (int)(width * 0.4);
        curColorLight = Globals.PADDLE_SPECIAL_COLOR_LIGHT;
        curColorDark = Globals.PADDLE_SPECIAL_COLOR_DARK;
    }



    /* ----------------------------------------------------------------------
     * Paddle Shooting
     * ---------------------------------------------------------------------- */
    /**
     * Add the special feature 'shooting' to the paddle. Called if the corresponding
     * special item was collected by the paddle.
     */
    public void AddShootingFeature(GUNSHOT_TYPE type)
    {
        if (type == GUNSHOT_TYPE.NORMAL)
        {
            curColorLight = Globals.SI_PADDLESHOOT_COLOR;
            curColorDark = Globals.SI_PADDLESHOOT_COLOR.darker().darker();
        }
        else if (type == GUNSHOT_TYPE.EXTENDED)
        {
            curColorLight = Globals.SI_PADDLESHOOTEX_COLOR;
            curColorDark = Globals.SI_PADDLESHOOTEX_COLOR.darker().darker();
        }
        shots[SHOT_LEFT].setShotType(type);
        shots[SHOT_RIGHT].setShotType(type);
        isShootingFeatureActive = true;
    }

    /*
     * Trigger a new shot if possible. Called if the user presses a mousebutton.
     */
    public void TriggerShot()
    {
        if (isShootingFeatureActive && !IsAShotVisible())
        {
            // special item is active
            // init left shot
            shots[SHOT_LEFT].x = this.x;
            shots[SHOT_LEFT].y = this.y - shots[SHOT_LEFT].height;
            shots[SHOT_LEFT].visible = true;
            // init right shot
            shots[SHOT_RIGHT].x = this.x + this.width - shots[SHOT_RIGHT].width;
            shots[SHOT_RIGHT].y = this.y - shots[SHOT_RIGHT].height;
            shots[SHOT_RIGHT].visible = true;
        }
    }

    /**
     * Check if a gunshot is visible.
     * @return true if one or both shots are currently visible, otherwise false.
     */
    public boolean IsAShotVisible()
    {
        return (shots[SHOT_LEFT].visible || shots[SHOT_RIGHT].visible);
    }

    /**
     * Checks if a gunshot hits a levelblock and handle collision.
     * @param level pointer to the current level.
     * @return true if one or more blocks were hit; false if no block was hit
     */
    private boolean HandleGunShotLevelBlockCollision(GameLevel level)
    {
        int row, column;
        boolean result = false;
        
        /* check collision of left shot with block */
        // calculate block grid position from left shot position
        if (shots[SHOT_LEFT].visible)
        {
            row = (int)((shots[SHOT_LEFT].y - Globals.BLOCK_AREA_TOP) / Globals.BLOCKSIZE_Y);
            column = (int)((shots[SHOT_LEFT].x + shots[SHOT_LEFT].width) / Globals.BLOCKSIZE_X);
            // check if at the shot's position there is actually an block
            if ( (row >= 0) && (row < Globals.NUM_BLOCKS_Y) &&
                 (column >= 0) && (column < Globals.NUM_BLOCKS_X) )
            {
                if (!level.IsBlockEmpty(column, row))
                {
                    level.BlockIsHit(column, row);
                    shots[SHOT_LEFT].hitBlock();
                    result = true;
                }
            }
        }

        /* check collision of right shot with block */
        // calculate block grid position from left shot position
        if (shots[SHOT_RIGHT].visible)
        {
            row = (int)((shots[SHOT_RIGHT].y - Globals.BLOCK_AREA_TOP) / Globals.BLOCKSIZE_Y);
            column = (int)(shots[SHOT_RIGHT].x / Globals.BLOCKSIZE_X);
            // check if at the shot's position there is actually an block
            if ( (row >= 0) && (row < Globals.NUM_BLOCKS_Y) &&
                 (column >= 0) && (column < Globals.NUM_BLOCKS_X) )
            {
                if (!level.IsBlockEmpty(column, row))
                {
                    level.BlockIsHit(column, row);
                    shots[SHOT_RIGHT].hitBlock();
                    result = true;
                }
            }
        }

        return result;
    }


    /**
     * Internal class representing one shot.
     */
    class GunShot {
        public double x,y;          // position of shot
        public int width, height;   // size of a shot
        public boolean visible;     // visibility / activity flag

        private GUNSHOT_TYPE type;  // type of bullet
        
        /**
         * Standard constructor. Shot inactive by default.
         */
        public GunShot()
        {
            this.visible = false;
            this.type = GUNSHOT_TYPE.NORMAL;
        }

        /**
         * Set the type of the shot/bullet.
         * @param newShottype
         */
        public void setShotType(GUNSHOT_TYPE newShottype)
        {
            type = newShottype;
            if (type == GUNSHOT_TYPE.NORMAL)
            {
                width = Globals.SI_PADDLESHOT_WIDTH;
                height = Globals.SI_PADDLESHOT_HEIGHT;
            }
            else if (type == GUNSHOT_TYPE.EXTENDED)
            {
                width = Globals.SI_PADDLESHOTEX_WIDTH;
                height = Globals.SI_PADDLESHOTEX_HEIGHT;
            }
        }

        public void hitBlock()
        {
            if (type == GUNSHOT_TYPE.NORMAL)
            {
                visible = false;
            }
            else
            {
                // extended shot is not destroyed when hitting a block
            }
        }

        /**
         * Draw the gunshot.
         * @param g Graphics object to draw on.
         */
        public void draw(Graphics2D g)
        {
            if (this.visible)
            {
                Color oldColor = g.getColor();
                g.setColor(Globals.SI_PADDLESHOOT_COLOR);
                g.fillRoundRect((int)x, (int)y, width, height, 5, 5);
                g.setColor(oldColor);
            }
        }
    }
}
